@@ -36,6 +36,7 @@ import sys
 import time
 
 from kazoo.exceptions import NoAuthError, NoNodeError, NotEmptyError
+from kazoo.handlers.threading import TimeoutError
 
 from .acl import ACLReader
 from .augumented_client import AugumentedClient
@@ -49,7 +50,7 @@ from .augumented_cmd import (
     Optional,
     Required,
 )
-from .copy import copy, CopyError
+from .copy import CopyError, Proxy
 from .watch_manager import get_watch_manager
 from .util import pretty_bytes, to_bool
 
@@ -70,12 +71,12 @@ class Shell(AugumentedCmd):
     def connected(f):
         def wrapped(self, args):
             if not self.connected:
-                print("Not connected.")
+                print("Not connected.", file=self._output)
             else:
                 try:
                     return f(self, args)
                 except NoAuthError as ex:
-                    print("Not authenticated.")
+                    print("Not authenticated.", file=self._output)
         wrapped.__doc__ = f.__doc__
         return wrapped
 
@@ -196,19 +197,57 @@ class Shell(AugumentedCmd):
     def do_cp(self, params):
         """
         copy from/to local/remote or remote/remote paths.
-        example:
+
+        src and dst can be any of:
+
+        /some/path (in the connected server)
+        file://<path>
+        zk://[user:passwd@]host/<path>
+        json://!some!path!backup.json/some/path
+
+        with a few restrictions. bare in mind the semantic differences
+        that znodes have with filesystem directories - so recursive copying
+        from znodes to an fs could lose data, but to a JSON file it would
+        work just fine.
+
+        examples:
+        cp /some/znode /backup/copy-znode  # local
         cp file://<path> zk://[user:passwd@]host/<path> <recursive> <overwrite> <async> <verbose>
+        cp /some/path json://!home!user!backup.json/ true true
         """
+
+        # default to zk://connected_host, if connected
+        src_connected_zk = dst_connected_zk = False
+        if self.connected:
+            zk_url = self._zk.zk_url()
+
+            # if these are local paths, make them absolute paths
+            if not re.match("^\w+://", params.src):
+                params.src = "%s%s" % (zk_url, self.abspath(params.src))
+                src_connected_zk = True
+
+            if not re.match("^\w+://", params.dst):
+                params.dst = "%s%s" % (zk_url, self.abspath(params.dst))
+                dst_connected_zk = True
+
         try:
-            copy(params.src,
-                 params.dst,
-                 params.recursive,
-                 params.overwrite,
-                 params.async,
-                 params.verbose)
+            src = Proxy.from_string(params.src, True)
+            if src_connected_zk:
+                src.need_client = False
+                src.client = self._zk
+
+            dst = Proxy.from_string(params.dst, None if params.overwrite else False)
+            if dst_connected_zk:
+                dst.need_client = False
+                dst.client = self._zk
+
+            src.copy(dst, params.recursive, params.async, params.verbose)
         except CopyError as ex:
             print(str(ex))
 
+    def complete_cp(self, cmd_param_text, full_cmd, start_idx, end_idx):
+        return self._complete_path(cmd_param_text, full_cmd)
+
     @connected
     @interruptible
     @ensure_params(Optional("path"), IntegerOptional("max_depth"))
@@ -482,7 +521,7 @@ server=%s""" % (self._zk.state,
                 self._zk._connection._xid,
                 self._zk.last_zxid,
                 self._zk._session_timeout,
-                self._zk._connection._socket.getpeername()))
+                self._zk._connection._socket.getpeername()), file=self._output)
 
     @ensure_params(Optional("host"))
     def do_mntr(self, params):
@@ -563,19 +602,19 @@ server=%s""" % (self._zk.state,
         self._exit(False)
 
     def _disconnect(self):
-        if self._zk: return
-
-        try: self._zk.stop()
-        except Exception: pass
+        if self._zk:
+            self._zk.stop()
+            self._zk = None
+        self.connected = False
 
     def _connect(self, hosts):
         self._disconnect()
-        self._zk = AugumentedClient(",".join(hosts),
-                                    read_only=self._read_only)
+        self._zk = AugumentedClient(",".join(hosts), read_only=self._read_only)
         try:
             self._zk.start(timeout=self._connect_timeout)
             self.connected = True
-        except Exception as ex: print("Failed to connect: %s" % (ex))
+        except TimeoutError as ex:
+            print("Failed to connect: %s" % (ex), file=self._output)
 
         self.update_curdir("/")
 
